/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  int AFwrI3 (AFILE *AFp, const float Fbuff[], int Nval)

Purpose:
  Write 24-bit integer data to an audio file (float input values)

Description:
  This routine writes a specified number of 24-bit integer samples to an audio
  file.  The input to this routine is a buffer of float values.

Parameters:
  <-  int AFwrI3
      Number of samples written.  If this value is less than Nval, an error
      has occurred.
   -> AFILE *AFp
      Audio file pointer for an audio file opened by AFopenWrite
   -> const float Fbuff[]
      Array of floats with the samples to be written
   -> int Nval
      Number of samples to be written

Author / revision:
  P. Kabal  Copyright (C) 1999

  $Revision: 1.7 $  $Date: 1999/06/12 19:37:01 $

-------------------------------------------------------------------------*/

static char rcsid [] = "$Id: AFwrI3.c 1.7 1999/06/12 AFsp-v5r2 $";

#include <libtsp/nucleus.h>
#include <libtsp/AFdataio.h>
#include <libtsp/AFpar.h>
#include <libtsp/UTtypes.h>

#define LW		FDL_INT24
#define MINV(a, b)	(((a) < (b)) ? (a) : (b))
#define NBBUF		8192
#define INT3_MAX	8388607
#define INT3_MIN	-8388608

#define FWRITE(buf,size,nv,fp)	(int) fwrite ((const char *) buf, \
					      (size_t) size, (size_t) nv, fp)


int
AFwrI3 (AFILE *AFp, const float Fbuff[], int Nval)

{
  int is, N, Nw, i, j, Hbo;
  int4_t Iv;
  unsigned char Buf[NBBUF];
  double g, Dv;
  unsigned char *cp;
  unsigned char t;

/* Write data to the audio file */
  Hbo = UTbyteOrder ();
  cp = (unsigned char *) &Iv;
  is = 0;
  g = AFp->ScaleF;
  while (is < Nval) {
    N = MINV (NBBUF / LW, Nval - is);
    for (i = 0, j = is; i < LW*N; i += LW, ++j) {
      Dv = g * Fbuff[j];
      if (Dv >= 0.0F) {
	Dv += 0.5F;
	if (Dv >= (double) INT3_MAX + 1.) {
	  ++AFp->Novld;
	  Dv = INT3_MAX;
	}
      }
      else {
	Dv += -0.5F;
	if (Dv <= (double) INT3_MIN - 1.) {
	  ++AFp->Novld;
	  Dv = INT3_MIN;
	}
      }
      Iv = 256 * ((int4_t) Dv);
      if (Hbo == DS_EL) {
	Buf[i] = cp[1];
	Buf[i+1] = cp[2];
	Buf[i+2] = cp[3];	/* Most significant byte */
      }
      else {
	Buf[i] = cp[0];		/* Most significant byte */
	Buf[i+1] = cp[1];
	Buf[i+2] = cp[2];
      }
      if (AFp->Swapb == DS_SWAP) {
	t = Buf[i+2]; Buf[i+2] = Buf[i]; Buf[i] = t;
      }
    }
    Nw = FWRITE (Buf, LW, N, AFp->fp);
    is += Nw;
    if (Nw < N)
      break;
  }

  return is;
}
