/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  int AFwrI4 (AFILE *AFp, const float Fbuff[], int Nval)

Purpose:
  Write 32-bit integer data to an audio file (float input values)

Description:
  This routine writes a specified number of 32-bit integer samples to an audio
  file.  The input to this routine is a buffer of float values.

Parameters:
  <-  int AFwrI4
      Number of samples written.  If this value is less than Nval, an error
      has occurred.
   -> AFILE *AFp
      Audio file pointer for an audio file opened by AFopenWrite
   -> const float Fbuff[]
      Array of floats with the samples to be written
   -> int Nval
      Number of samples to be written

Author / revision:
  P. Kabal  Copyright (C) 1999

  $Revision: 1.5 $  $Date: 1999/06/11 22:41:24 $

-------------------------------------------------------------------------*/

static char rcsid [] = "$Id: AFwrI4.c 1.5 1999/06/11 AFsp-v5r2 $";

#include <libtsp/AFdataio.h>
#include <libtsp/AFpar.h>
#include <libtsp/UTtypes.h>

#define LW		FDL_INT32
#define MINV(a, b)	(((a) < (b)) ? (a) : (b))
#define NBBUF		8192

#define FWRITE(buf,size,nv,fp)	(int) fwrite ((const char *) buf, \
					      (size_t) size, (size_t) nv, fp)


int
AFwrI4 (AFILE *AFp, const float Fbuff[], int Nval)

{
  int is, N, Nw, i;
  int4_t Buf[NBBUF/LW];
  double g, Dv;
  unsigned char *cp;
  unsigned char t;

/* Write data to the audio file */
  is = 0;
  g = AFp->ScaleF;
  while (is < Nval) {
    N = MINV (NBBUF / LW, Nval - is);
    for (i = 0; i < N; ++i) {
      Dv = g * Fbuff[i+is];
      if (Dv >= 0.0) {
	Dv += 0.5;
	if (Dv >= (double) INT4_MAX + 1.) {
	  ++AFp->Novld;
	  Dv = INT4_MAX;
	}
      }
      else {
	Dv += -0.5;
	if (Dv <= (double) (INT4_MIN) - 1.) {
	  ++AFp->Novld;
	  Dv = INT4_MIN;
	}
      }
      Buf[i] = (int4_t) Dv;
      if (AFp->Swapb == DS_SWAP) {
	cp = (unsigned char *) &Buf[i];
	t = cp[3]; cp[3] = cp[0]; cp[0] = t;
	t = cp[2]; cp[2] = cp[1]; cp[1] = t;
      }
    }
    Nw = FWRITE (Buf, LW, N, AFp->fp);
    is += Nw;
    if (Nw < N)
      break;
  }

  return is;
}
