/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  AFILE *AFsetWrite (FILE *fp, int Ftype, struct AF_write *AFw)

Purpose:
  Set up the parameters in an audio file structure for wrinting

Description:
  This routine checks the input parameters and puts them into a new audio file
  structure.

  This routine checks for a "sample_rate:" record in the information string.
  This record can specify a non-integer sampling frequency.

Parameters:
  <-  AFILE *AFsetWrite
      Audio file pointer for the audio file.  This routine allocates the space
      for this structure.
   -> FILE *fp
      File pointer for the audio file
   -> int Ftype
      File type: FT_NH, FT_AU, FT_WAVE, or FT_AIFF_C
  <-> struct AF_write *AFw
      Structure with data and file parameters

Author / revision:
  P. Kabal  Copyright (C) 1999

  $Revision: 1.34 $  $Date: 1999/06/07 16:29:23 $

-------------------------------------------------------------------------*/

static char rcsid [] = "$Id: AFsetWrite.c 1.34 1999/06/07 AFsp-v5r2 $";

#include <assert.h>
#include <string.h>

#include <libtsp.h>
#include <libtsp/nucleus.h>
#include <libtsp/AFdataio.h>
#include <libtsp/AFheader.h>
#include <libtsp/AFmsg.h>
#define AF_DATA_LENGTHS
#include <libtsp/AFpar.h>

static void
AF_setInfo (const struct AF_info *Hinfo, struct AF_info *AFinfo);


AFILE *
AFsetWrite (FILE *fp, int Ftype, struct AF_write *AFw)

{
  AFILE *AFp;
  long int Start;
  int ErrCode;

  assert (Ftype == FT_WAVE || Ftype == FT_AU ||
	  Ftype == FT_AIFF_C || Ftype == FT_NH);
  assert (AFw->DFormat.Format > 0 && AFw->DFormat.Format < NFD);
  assert (AFw->Nchan > 0L);

/* Warnings */
  if (AFw->Sfreq <= 0.0)
    UTwarn ("AFsetWrite - %s", AFM_NPSFreq);

/* Set the start of data */
  if (FLseekable (fp)) {
    ErrCode = 0;
    Start = AFtell (fp, &ErrCode);
    if (ErrCode)
      return NULL;
  }
  else
    Start = 0L;

/* Set the parameters for file access */
  AFp = (AFILE *) UTmalloc (sizeof (AFILE));
  AFp->fp = fp;
  AFp->Op = FO_WO;
  AFp->Error = AF_NOERR;
  AFp->Ftype = Ftype;
  AFp->Format = AFw->DFormat.Format;
  if (AF_DL[AFw->DFormat.Format] <= 1)
    AFp->Swapb = DS_NATIVE;
  else
    AFp->Swapb = UTswapCode (AFw->DFormat.Swapb);
  AFp->Sfreq = AFw->Sfreq;
  AFp->ScaleF = AFw->DFormat.ScaleF;
  AFp->Nchan = AFw->Nchan;
  AFp->Start = Start;
  AFp->Isamp = 0L;
  AFp->Nsamp = 0L;
  AFp->Novld = 0L;
  AF_setInfo (&AFw->Hinfo, &AFp->Hinfo);

  return AFp;
}

/* Fill in the AFsp Information structure */


static void
AF_setInfo (const struct AF_info *Hinfo, struct AF_info *AFinfo)

{
  int N;

  if (Hinfo == NULL || Hinfo->N <= 0) {
    AFinfo->Info = NULL;
    AFinfo->N = 0;
    AFinfo->Nmax = 0;
  }
  else {
    N = Hinfo->N;
    if (Hinfo->Info[N-1] != '\0')
      ++N;
    AFinfo->Info = (char *) UTmalloc (N);
    memcpy (AFinfo->Info, Hinfo->Info, Hinfo->N);
    AFinfo->Info[N-1] = '\0';	/* Make sure there is a terminating null */
    AFinfo->N = N;
    AFinfo->Nmax = N;
  }

  return;
}
